@@ -57,6 +57,7 @@ glue_solve_gen::apply(Mat<eT>& out, const Base<eT,T1>& A_expr, const Base<eT,T2>
   const bool allow_ugly   = bool(flags & solve_opts::flag_allow_ugly  );
   const bool likely_sympd = bool(flags & solve_opts::flag_likely_sympd);
   const bool refine       = bool(flags & solve_opts::flag_refine      );
+  const bool no_trimat    = bool(flags & solve_opts::flag_no_trimat   );
   
   arma_extra_debug_print("glue_solve_gen::apply(): enabled flags:");
   
@@ -68,10 +69,11 @@ glue_solve_gen::apply(Mat<eT>& out, const Base<eT,T1>& A_expr, const Base<eT,T2>
   if(allow_ugly  )  { arma_extra_debug_print("allow_ugly");   }
   if(likely_sympd)  { arma_extra_debug_print("likely_sympd"); }
   if(refine      )  { arma_extra_debug_print("refine");       }
+  if(no_trimat   )  { arma_extra_debug_print("no_trimat");    }
   
-  arma_debug_check( (fast     && equilibrate ), "solve(): options 'fast' and 'equilibrate' are mutually exclusive"        );
-  arma_debug_check( (fast     && refine      ), "solve(): options 'fast' and 'refine' are mutually exclusive"             );
-  arma_debug_check( (no_sympd && likely_sympd), "solve(): options 'no_sympd' and 'likely_sympd' are mutually exclusive"   );
+  arma_debug_check( (fast     && equilibrate ), "solve(): options 'fast' and 'equilibrate' are mutually exclusive"      );
+  arma_debug_check( (fast     && refine      ), "solve(): options 'fast' and 'refine' are mutually exclusive"           );
+  arma_debug_check( (no_sympd && likely_sympd), "solve(): options 'no_sympd' and 'likely_sympd' are mutually exclusive" );
   
   T    rcond  = T(0);
   bool status = false;
@@ -86,19 +88,26 @@ glue_solve_gen::apply(Mat<eT>& out, const Base<eT,T1>& A_expr, const Base<eT,T2>
     uword KU = 0;
     
     #if defined(ARMA_OPTIMISE_SOLVE_BAND)
-      const bool is_band  = ((no_band == false) && (auxlib::crippled_lapack(A) == false)) ? band_helper::is_band(KL, KU, A, uword(32)) : false;
+      const bool is_band  = (no_band || auxlib::crippled_lapack(A)) ? false : band_helper::is_band(KL, KU, A, uword(32));
     #else
       const bool is_band  = false;
     #endif
     
+    const bool is_triu = (no_trimat || refine || equilibrate || likely_sympd || is_band           ) ? false : trimat_helper::is_triu(A);
+    const bool is_tril = (no_trimat || refine || equilibrate || likely_sympd || is_band || is_triu) ? false : trimat_helper::is_tril(A);
+    
     #if defined(ARMA_OPTIMISE_SOLVE_SYMPD)
-      const bool try_sympd = ((no_sympd == false) && (auxlib::crippled_lapack(A) == false) && (is_band == false)) ? (likely_sympd ? true : sympd_helper::guess_sympd(A)) : false;
+      const bool try_sympd = (no_sympd || auxlib::crippled_lapack(A) || is_band || is_triu || is_tril) ? false : (likely_sympd ? true : sympd_helper::guess_sympd(A));
     #else
       const bool try_sympd = false;
     #endif
     
     if(fast)
       {
+      // fast mode: solvers without refinement and without rcond estimate
+      
+      arma_extra_debug_print("glue_solve_gen::apply(): fast mode");
+      
       if(is_band)
         {
         if( (KL == 1) && (KU == 1) )
@@ -115,6 +124,16 @@ glue_solve_gen::apply(Mat<eT>& out, const Base<eT,T1>& A_expr, const Base<eT,T2>
           }
         }
       else
+      if(is_triu || is_tril)
+        {
+        if(is_triu)  { arma_extra_debug_print("glue_solve_gen::apply(): fast + upper triangular matrix"); }
+        if(is_tril)  { arma_extra_debug_print("glue_solve_gen::apply(): fast + lower triangular matrix"); }
+        
+        const uword layout = (is_triu) ? uword(0) : uword(1);
+        
+        status = auxlib::solve_trimat_fast(out, A, B_expr.get_ref(), layout);
+        }
+      else
       if(try_sympd)
         {
         arma_extra_debug_print("glue_solve_gen::apply(): fast + try_sympd");
@@ -137,22 +156,18 @@ glue_solve_gen::apply(Mat<eT>& out, const Base<eT,T1>& A_expr, const Base<eT,T2>
         status = auxlib::solve_square_fast(out, A, B_expr.get_ref());  // A is overwritten
         }
       }
-    else  // solve with refinement and provide rcond estimate
+    else
+    if(refine || equilibrate)
       {
+      // refine mode: solvers with refinement and with rcond estimate
+      
+      arma_extra_debug_print("glue_solve_gen::apply(): refine mode");
+      
       if(is_band)
         {
-        if( (KL == 1) && (KU == 1) && (equilibrate == false) )
-          {
-          arma_extra_debug_print("glue_solve_gen::apply(): refine + tridiagonal");
-          
-          status = auxlib::solve_tridiag_refine(out, rcond, A, B_expr, allow_ugly);
-          }
-        else
-          {
-          arma_extra_debug_print("glue_solve_gen::apply(): refine + band");
-          
-          status = auxlib::solve_band_refine(out, rcond, A, KL, KU, B_expr, equilibrate, allow_ugly);
-          }
+        arma_extra_debug_print("glue_solve_gen::apply(): refine + band");
+        
+        status = auxlib::solve_band_refine(out, rcond, A, KL, KU, B_expr, equilibrate, allow_ugly);
         }
       else
       if(try_sympd)
@@ -177,9 +192,51 @@ glue_solve_gen::apply(Mat<eT>& out, const Base<eT,T1>& A_expr, const Base<eT,T2>
         status = auxlib::solve_square_refine(out, rcond, A, B_expr, equilibrate, allow_ugly);  // A is overwritten
         }
       }
+    else
+      {
+      // default mode: solvers without refinement but with rcond estimate
+      
+      arma_extra_debug_print("glue_solve_gen::apply(): default mode");
+      
+      if(is_band)
+        {
+        arma_extra_debug_print("glue_solve_gen::apply(): rcond + band");
+        
+        status = auxlib::solve_band_rcond(out, rcond, A, KL, KU, B_expr.get_ref(), allow_ugly);
+        }
+      else
+      if(is_triu || is_tril)
+        {
+        if(is_triu)  { arma_extra_debug_print("glue_solve_gen::apply(): rcond + upper triangular matrix"); }
+        if(is_tril)  { arma_extra_debug_print("glue_solve_gen::apply(): rcond + lower triangular matrix"); }
+        
+        const uword layout = (is_triu) ? uword(0) : uword(1);
+        
+        status = auxlib::solve_trimat_rcond(out, rcond, A, B_expr.get_ref(), layout, allow_ugly);
+        }
+      else
+      if(try_sympd)
+        {
+        status = auxlib::solve_sympd_rcond(out, rcond, A, B_expr.get_ref(), allow_ugly);  // A is overwritten
+        
+        if(status == false)
+          {
+          arma_extra_debug_print("glue_solve_gen::apply(): auxlib::solve_sympd_rcond() failed; retrying");
+          
+          // auxlib::solve_sympd_rcond() may have failed because A isn't really sympd
+          A = A_expr.get_ref();
+          status = auxlib::solve_square_rcond(out, rcond, A, B_expr.get_ref(), allow_ugly);  // A is overwritten
+          }
+        }
+      else
+        {
+        status = auxlib::solve_square_rcond(out, rcond, A, B_expr.get_ref(), allow_ugly);  // A is overwritten
+        }
+      }
     
     
-    if( (status == true) && (rcond > T(0)) && (rcond <= (T(0.5)*std::numeric_limits<T>::epsilon())) )
+    
+    if( (status == true) && (rcond > T(0)) && (rcond < auxlib::epsilon_lapack(A)) )
       {
       arma_debug_warn("solve(): solution computed, but system seems singular to working precision (rcond: ", rcond, ")");
       }
@@ -209,8 +266,8 @@ glue_solve_gen::apply(Mat<eT>& out, const Base<eT,T1>& A_expr, const Base<eT,T2>
     {
     arma_extra_debug_print("glue_solve_gen::apply(): detected non-square system");
     
-    if(equilibrate)   { arma_debug_warn( "solve(): option 'equilibrate' ignored for non-square matrix" );  }
-    if(refine)        { arma_debug_warn( "solve(): option 'refine' ignored for non-square matrix" );       }
+    if(equilibrate)   { arma_debug_warn( "solve(): option 'equilibrate' ignored for non-square matrix"  ); }
+    if(refine)        { arma_debug_warn( "solve(): option 'refine' ignored for non-square matrix"       ); }
     if(likely_sympd)  { arma_debug_warn( "solve(): option 'likely_sympd' ignored for non-square matrix" ); }
     
     if(fast)
@@ -242,6 +299,91 @@ glue_solve_gen::apply(Mat<eT>& out, const Base<eT,T1>& A_expr, const Base<eT,T2>
 // glue_solve_tri
 
 
+template<typename T1, typename T2>
+inline
+void
+glue_solve_tri_default::apply(Mat<typename T1::elem_type>& out, const Glue<T1,T2,glue_solve_tri_default>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  const bool status = glue_solve_tri_default::apply( out, X.A, X.B, X.aux_uword );
+  
+  if(status == false)
+    {
+    arma_stop_runtime_error("solve(): solution not found");
+    }
+  }
+
+
+
+template<typename eT, typename T1, typename T2>
+inline
+bool
+glue_solve_tri_default::apply(Mat<eT>& actual_out, const Base<eT,T1>& A_expr, const Base<eT,T2>& B_expr, const uword flags)
+  {
+  arma_extra_debug_sigprint();
+  
+  typedef typename get_pod_type<eT>::result T;
+  
+  const bool triu       = bool(flags & solve_opts::flag_triu);
+  const bool tril       = bool(flags & solve_opts::flag_tril);
+  const bool allow_ugly = false;
+  
+  arma_extra_debug_print("glue_solve_tri_default::apply(): enabled flags:");
+  
+  if(triu)  { arma_extra_debug_print("triu"); }
+  if(tril)  { arma_extra_debug_print("tril"); }
+  
+  const quasi_unwrap<T1> U(A_expr.get_ref());
+  const Mat<eT>& A     = U.M;
+  
+  arma_debug_check( (A.is_square() == false), "solve(): matrix marked as triangular must be square sized" );
+  
+  const uword layout   = (triu) ? uword(0) : uword(1);
+  const bool  is_alias = U.is_alias(actual_out);
+  
+  T    rcond  = T(0);
+  bool status = false;
+  
+  Mat<eT>  tmp;
+  Mat<eT>& out = (is_alias) ? tmp : actual_out;
+  
+  status = auxlib::solve_trimat_rcond(out, rcond, A, B_expr.get_ref(), layout, allow_ugly);  // A is not modified
+  
+  if( (status == true) && (rcond > T(0)) && (rcond < auxlib::epsilon_lapack(A)) )
+    {
+    arma_debug_warn("solve(): solution computed, but system seems singular to working precision (rcond: ", rcond, ")");
+    }
+  
+  
+  if(status == false)
+    {
+    arma_extra_debug_print("glue_solve_tri::apply(): solving rank deficient system");
+    
+    if(rcond > T(0))
+      {
+      arma_debug_warn("solve(): system seems singular (rcond: ", rcond, "); attempting approx solution");
+      }
+    else
+      {
+      arma_debug_warn("solve(): system seems singular; attempting approx solution");
+      }
+    
+    Mat<eT> triA = (triu) ? trimatu(A) : trimatl(A);  // trimatu() and trimatl() return the same type
+    
+    status = auxlib::solve_approx_svd(out, triA, B_expr.get_ref());  // triA is overwritten
+    }
+  
+  
+  if(status == false)  { out.soft_reset(); }
+  
+  if(is_alias)  { actual_out.steal_mem(out); }
+  
+  return status;
+  }
+
+
+
 template<typename T1, typename T2>
 inline
 void
@@ -262,17 +404,21 @@ glue_solve_tri::apply(Mat<typename T1::elem_type>& out, const Glue<T1,T2,glue_so
 template<typename eT, typename T1, typename T2>
 inline
 bool
-glue_solve_tri::apply(Mat<eT>& out, const Base<eT,T1>& A_expr, const Base<eT,T2>& B_expr, const uword flags)
+glue_solve_tri::apply(Mat<eT>& actual_out, const Base<eT,T1>& A_expr, const Base<eT,T2>& B_expr, const uword flags)
   {
   arma_extra_debug_sigprint();
   
+  typedef typename get_pod_type<eT>::result T;
+  
   const bool fast         = bool(flags & solve_opts::flag_fast        );
   const bool equilibrate  = bool(flags & solve_opts::flag_equilibrate );
   const bool no_approx    = bool(flags & solve_opts::flag_no_approx   );
   const bool triu         = bool(flags & solve_opts::flag_triu        );
   const bool tril         = bool(flags & solve_opts::flag_tril        );
+  const bool allow_ugly   = bool(flags & solve_opts::flag_allow_ugly  );
   const bool likely_sympd = bool(flags & solve_opts::flag_likely_sympd);
   const bool refine       = bool(flags & solve_opts::flag_refine      );
+  const bool no_trimat    = bool(flags & solve_opts::flag_no_trimat   );
   
   arma_extra_debug_print("glue_solve_tri::apply(): enabled flags:");
   
@@ -281,13 +427,18 @@ glue_solve_tri::apply(Mat<eT>& out, const Base<eT,T1>& A_expr, const Base<eT,T2>
   if(no_approx   )  { arma_extra_debug_print("no_approx");    }
   if(triu        )  { arma_extra_debug_print("triu");         }
   if(tril        )  { arma_extra_debug_print("tril");         }
+  if(allow_ugly  )  { arma_extra_debug_print("allow_ugly");   }
   if(likely_sympd)  { arma_extra_debug_print("likely_sympd"); }
   if(refine      )  { arma_extra_debug_print("refine");       }
+  if(no_trimat   )  { arma_extra_debug_print("no_trimat");    }
   
-  bool status = false;
+  if(no_trimat || equilibrate || refine)
+    {
+    const uword mask = ~(solve_opts::flag_triu | solve_opts::flag_tril);
+    
+    return glue_solve_gen::apply(actual_out, ((triu) ? trimatu(A_expr.get_ref()) : trimatl(A_expr.get_ref())), B_expr, (flags & mask));
+    }
   
-  if(equilibrate)   { arma_debug_warn("solve(): option 'equilibrate' ignored for triangular matrix");  }
-  if(refine)        { arma_debug_warn("solve(): option 'refine' ignored for triangular matrix");       }
   if(likely_sympd)  { arma_debug_warn("solve(): option 'likely_sympd' ignored for triangular matrix"); }
   
   const quasi_unwrap<T1> U(A_expr.get_ref());
@@ -295,19 +446,27 @@ glue_solve_tri::apply(Mat<eT>& out, const Base<eT,T1>& A_expr, const Base<eT,T2>
   
   arma_debug_check( (A.is_square() == false), "solve(): matrix marked as triangular must be square sized" );
   
-  const uword layout = (triu) ? uword(0) : uword(1);
+  const uword layout   = (triu) ? uword(0) : uword(1);
+  const bool  is_alias = U.is_alias(actual_out);
   
-  if(U.is_alias(out))
+  T    rcond  = T(0);
+  bool status = false;
+  
+  Mat<eT>  tmp;
+  Mat<eT>& out = (is_alias) ? tmp : actual_out;
+  
+  if(fast)
     {
-    Mat<eT> tmp;
-    
-    status = auxlib::solve_tri(tmp, A, B_expr.get_ref(), layout);  // A is not modified
-    
-    out.steal_mem(tmp);
+    status = auxlib::solve_trimat_fast(out, A, B_expr.get_ref(), layout);  // A is not modified
     }
   else
     {
-    status = auxlib::solve_tri(out, A, B_expr.get_ref(), layout);  // A is not modified
+    status = auxlib::solve_trimat_rcond(out, rcond, A, B_expr.get_ref(), layout, allow_ugly);  // A is not modified
+    }
+  
+  if( (status == true) && (rcond > T(0)) && (rcond < auxlib::epsilon_lapack(A)) )
+    {
+    arma_debug_warn("solve(): solution computed, but system seems singular to working precision (rcond: ", rcond, ")");
     }
   
   
@@ -315,7 +474,14 @@ glue_solve_tri::apply(Mat<eT>& out, const Base<eT,T1>& A_expr, const Base<eT,T2>
     {
     arma_extra_debug_print("glue_solve_tri::apply(): solving rank deficient system");
     
-    arma_debug_warn("solve(): system seems singular; attempting approx solution");
+    if(rcond > T(0))
+      {
+      arma_debug_warn("solve(): system seems singular (rcond: ", rcond, "); attempting approx solution");
+      }
+    else
+      {
+      arma_debug_warn("solve(): system seems singular; attempting approx solution");
+      }
     
     Mat<eT> triA = (triu) ? trimatu(A) : trimatl(A);  // trimatu() and trimatl() return the same type
     
@@ -325,6 +491,8 @@ glue_solve_tri::apply(Mat<eT>& out, const Base<eT,T1>& A_expr, const Base<eT,T2>
   
   if(status == false)  { out.soft_reset(); }
   
+  if(is_alias)  { actual_out.steal_mem(out); }
+  
   return status;
   }
 
